﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.ComponentModel;
using System.Drawing.Design;
using AjaxSnippets.Design;
using System.Web.UI.WebControls;

namespace AjaxSnippets
{
    /// <summary>
    /// Used in a SnippetCallback to define a mapping between a service method argument
    /// and a client dom element such as an input field. The mapping also defines the name of a 
    /// client function that will be used to extract a value from a DOM element to be used as 
    /// the service method argument value
    /// </summary>
    public class CallbackParameterMapping
    {
        /// <summary>
        /// The argument name of the web service method
        /// that will have its value set with the value that is extracted
        /// from the server control markup
        /// </summary>
        [Browsable(true)]
        [Description("The argument name of the web service method")]
        public string ArgumentName { get; set; }

        /// <summary>
        /// The ControlID of a server control that will have its value
        /// mapped to the service method argument, use ClientID if your
        /// not mapping to a server control
        /// </summary>
        [Browsable(true)]
        [IDReferenceProperty]
        [TypeConverter(typeof(ControlIDConverter))]
        [Description("The ID of a server control that will have its value mapped to the service method argument, use ClientID if not mapping to a server control")]
        public string ControlID { get; set; }
        /// <summary>
        /// The ClientID of the rendered server controls markup, use
        /// ControlID if mapping to a server control
        /// </summary>
        [Browsable(true)]
        [Description("The ClientID of the rendered server controls markup, use ControlID if mapping to a server control")]
        public string ClientID { get; set; }

        /// <summary>
        /// A function on the client that extracts the value
        /// from the server control markup, the functions are defined in the client
        /// class AjaxSnippets.ValueExtractors, the prototype can be extended to
        /// support custom extractors
        /// 
        /// If a ControlID is provided then the will automatically be determined from
        /// defaults or configuration
        /// </summary>
        [Browsable(true)]
        [Description("A function in the client class AjaxSnippets.ValueExtractors that extracts the value from the server control markup")]
        public string ClientValueExtractorFunctionName { get; set; }

        public override string ToString()
        {
            string mapsTo = String.IsNullOrEmpty(ControlID) ? ClientID : ControlID;

            return String.Format
            (
                "{0} maps to {1}",
                String.IsNullOrEmpty(mapsTo) ? "Nothing" : mapsTo,
                String.IsNullOrEmpty(ArgumentName) ? "nothing" : ArgumentName
            );
        }
        public string ToJson()
        {
            return "{" + String.Format("'param':'{0}','id':'{1}','func':'{2}'", ArgumentName, ClientID, ClientValueExtractorFunctionName) + "}";
        }
    }
}
